> module Display (display,tick) where
> import Graphics.Rendering.OpenGL
> import Graphics.Rendering.OpenGL.GL.StateVar
> import Graphics.UI.GLUT
> import Data.IORef
> import Consts
> import Entities
> import Colors 
> import Treestructs2

draw a cube

> cube :: GLfloat-> IO ()
> cube w = do 
>   renderPrimitive Quads $ do
>     vertex $ Vertex2 w w
>     vertex $ Vertex2 w (-w) 
>     vertex $ Vertex2 (-w) (-w)
>     vertex $ Vertex2 (-w) w 


> delta :: GLfloat
> delta = 0.3;

> drawTree :: Tree Int -> Vertex2 GLfloat -> GLfloat -> IO()
> drawTree (Fork _ ta tb) (Vertex2 x y) s = do
>                             renderPrimitive Lines $ do
>                                                     vertex $ Vertex2 x y
>                                                     vertex $ Vertex2 x (y-delta)
>                                                     vertex $ Vertex2 (x-s) (y-delta)
>                                                     vertex $ Vertex2 (x+s) (y-delta)
>                             drawTree ta (Vertex2 (x-s) (y-delta)) (s/2)
>                             drawTree tb (Vertex2 (x+s) (y-delta)) (s/2)
> drawTree (Leaf a) (Vertex2 x y) s = do
>                             renderPrimitive Lines $ do
>                                                     vertex $ Vertex2 x y
>                                                     vertex $ Vertex2 x (y-delta)
>                             renderPrimitive Lines $ do
>                                                     vertex $ Vertex2 (x-2*s) 0
>                                                     vertex $ Vertex2 (x+2*s) 0
> drawTree (Player a) (Vertex2 x y) s = do
>                             renderPrimitive Lines $ do
>                                                     color red
>                                                     vertex $ Vertex2 x y
>                                                     vertex $ Vertex2 x (y-delta)
>                             renderPrimitive Lines $ do
>                                                     vertex $ Vertex2 (x-2*s) 0
>                                                     vertex $ Vertex2 (x+2*s) 0
>                                                     color white


> offset :: GLfloat
> offset = 1 - (1/2)^^(maxHeight)

> display :: IORef(GameState) -> IO ()
> display gamestate = do 
>   clear [ColorBuffer]
>   loadIdentity
>   preservingMatrix $ do 
>                     GameState{entity=Entity{pos=(px,py)}} <-  get (gamestate)
>                     scale (0.7::GLfloat) 0.7 0.7
>                     translate $ Vector3 (-1.3::GLfloat) (-1.3) 0.0

                     drawTree (makeTree (px+(k1 px)*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+(k1 px)*2.4) 2.3) 0.6
                     drawTree (makeTree (px+(k2 px)*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+(k2 px)*2.4) 2.3) 0.6
                     drawTree (makeTree (px+(k3 px)*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+(k3 px)*2.4) 2.3) 0.6

>                     renderPrimitive Lines $ do
>                                                     vertex $ Vertex2 (-10::GLfloat) 2.3
>                                                     vertex $ Vertex2 (10::GLfloat) 2.3

> {-                    drawTree (makeTree (px+4*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))-4*2.4) 2.3) 0.6
>                     drawTree (makeTree (px+3*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))-3*2.4) 2.3) 0.6
>                     drawTree (makeTree (px+2*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))-2*2.4) 2.3) 0.6
>                     drawTree (makeTree (px+2^maxHeight)) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))-2.4) 2.3) 0.6
>                     drawTree (makeTree px)              (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+0.0) 2.3) 0.6
>                     drawTree (makeTree (px-2^maxHeight)) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+2.4) 2.3) 0.6
>                     drawTree (makeTree (px-2*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+2*2.4) 2.3) 0.6
>                     drawTree (makeTree (px-3*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+3*2.4) 2.3) 0.6
>                     drawTree (makeTree (px-4*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+4*2.4) 2.3) 0.6
>                     print px-}

>                     drawTree (makeTree (px-(k1 px)*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+(k1 px)*2.4) 2.3) 0.6
>                     drawTree (makeTree (px-(k2 px)*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+(k2 px)*2.4) 2.3) 0.6
>                     drawTree (makeTree (px-(k3 px)*(2^maxHeight))) (Vertex2 (1.3+offset-(fromIntegral px)*(0.5**(fromIntegral (maxHeight-1)))+(k3 px)*2.4) 2.3) 0.6

>                     swapBuffers
>                              where 
>                                k1 x = fromIntegral (x `div` (2^(maxHeight))) -1
>                                k2 x = fromIntegral (x `div` (2^(maxHeight))) 
>                                k3 x = fromIntegral (x `div` (2^(maxHeight))) +1

  

> tick :: IORef (GameState) -> IO()
> tick gamestate = do
>   postRedisplay Nothing
>   addTimerCallback msInterval (tick gamestate)
